import FreeCAD as App
import Part
from freecad.{{ cookiecutter.base_package }}.icon import get_icon_path
from {{ cookiecutter.base_package }}.part import Box


class AddBoxCommand:
    """
    Command to add a Box.
    """

    NAME = 'AddBox'

    def Activated(self):
        document = App.ActiveDocument
        if not document:
            document = App.newDocument()
        box = Box.make()
        Part.show(box)

    def IsActive(self):
        return True

    def GetResources(self):
        return {
            'Pixmap': get_icon_path('Box.svg'),
            'MenuText': 'Add Box',
            'ToolTip': 'Add Box'
        }




import os

from .execute_command import execute_command


def handle_test_command(base_package: str,
                        root_of_git_repository: str,
                        with_coverage: bool = False) -> None:
    test_command = 'pytest --color=yes'
    if with_coverage:
        test_command += ' --cov {}'.format(base_package)
    test_dir = os.path.join(root_of_git_repository, 'tests')
    test_command += ' ' + test_dir
    execute_command(test_command)
    if with_coverage:
        execute_command('coverage html')
        print('Coverage report generated in htmlcov/ directory.')
        print('To view, open htmlcov/index.html in a web browser, or run:\n')
        print('    osewb browse coverage\n')



import os
import webbrowser


def handle_browse_command(root_of_git_repository: str,
                          browse_subcommand: str) -> None:
    if browse_subcommand == 'docs':
        path_to_index_html = os.path.join(
            root_of_git_repository, 'docs/_build/index.html')
        if not os.path.isfile(path_to_index_html):
            print('No index.html found in {}'.format(path_to_index_html))
            print('To build the documentation, run:\n')
            print('    osewb docs\n')
        else:
            webbrowser.open(path_to_index_html)
    elif browse_subcommand == 'coverage':
        path_to_index_html = os.path.join(
            root_of_git_repository, 'htmlcov/index.html')
        if not os.path.isfile(path_to_index_html):
            print('No index.html found in {}'.format(path_to_index_html))
            print('To build the coverage report, run:\n')
            print('    osewb test --coverage\n')
        else:
            webbrowser.open(path_to_index_html)



import os
from pathlib import Path

from .execute_command import execute_command


def handle_lint_command(root_of_git_repository: str,
                        should_fix: bool = False) -> None:
    current_dir = Path(os.path.dirname(os.path.realpath(__file__)))
    pycodestyle_config = current_dir.joinpath('.pycodestyle').resolve()
    if should_fix:
        execute_command('isort --recursive {}'.format(root_of_git_repository))
        execute_command('autopep8 ' +
                        '--global-config {} '.format(pycodestyle_config) +
                        '--in-place ' +
                        '--aggressive ' +
                        '--recursive {}'.format(root_of_git_repository))
        return None
    flake8_config = current_dir.joinpath('.flake8').resolve()
    flake8_code = execute_command(
        'flake8 --config {} {}'.format(flake8_config, root_of_git_repository), exit_on_non_zero_code=False)
    mypy_config = current_dir.joinpath('.mypy.ini').resolve()
    mypy_code = execute_command('mypy --config-file {} {}'.format(mypy_config, root_of_git_repository), env={
        'MYPY_FORCE_COLOR': '1'
    }, exit_on_non_zero_code=False)
    return_code_pairs = [('flake8', flake8_code), ('mypy', mypy_code)]
    was_lint_error = any([pair[1] for pair in return_code_pairs])
    if was_lint_error:
        print('\nLint errors detected from the following command(s):')
        commands_with_errors = [pair[0]
                                for pair in return_code_pairs if pair[1]]
        print('    {}\n'.format(', '.join(commands_with_errors)))
        exit(1)
    print('No lint warnings or errors detected!')



import FreeCAD as App
import FreeCADGui as Gui

from .icon import get_icon_path
from .{{ cookiecutter.command_registry_filename }} import register_commands


class {{ cookiecutter.workbench_class_name }}(Gui.Workbench):
    """
    {{ cookiecutter.machine_display_name }} Workbench
    """
    MenuText = '{{ cookiecutter.machine_title }}'
    ToolTip = \
        'A workbench for designing {{ cookiecutter.machine_display_name }} machines by Open Source Ecology'
    Icon = get_icon_path('Box.svg')

    def Initialize(self):
        """
        Executed when FreeCAD starts
        """
        main_toolbar, main_menu = register_commands()

        self.appendToolbar('{{ cookiecutter.machine_title }}', main_toolbar)
        self.appendMenu('{{ cookiecutter.machine_title }}', main_menu)

    def Activated(self):
        """
        Executed when workbench is activated.
        """
        if not(App.ActiveDocument):
            App.newDocument()

    def Deactivated(self):
        """
        Executed when workbench is deactivated.
        """
        pass

    def GetClassName(self):
        return 'Gui::PythonWorkbench'


Gui.addWorkbench({{ cookiecutter.workbench_class_name }}())



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

import json
# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
import os
import sys

from osewb.docs import conf

sys.path.insert(0, os.path.abspath('.'))
sys.path.insert(0, os.path.abspath('../'))

# Shared base configuration for OSE workbench documentation.
# https://github.com/gbroques/ose-workbench-platform/blob/master/osewb/docs/conf.py
print("============================================")
print("             SPHINX BASE CONFIG             ")
print("============================================")
print(json.dumps(conf, indent=4))
print("============================================")


def run_apidoc(app):
    """Generate API documentation"""
    from sphinx.ext import apidoc
    max_depth = '1'
    apidoc.main([
        '../{{ cookiecutter.base_package }}',
        '-o', '{{ cookiecutter.base_package }}',
        '-d', max_depth,
        '--templatedir=_templates/',
        '--force',
        '--no-toc'
    ])


def setup(app):
    app.connect('builder-inited', run_apidoc)


# -- Project information -----------------------------------------------------

project = '{{ cookiecutter.workbench_title }}'
copyright = '{{ cookiecutter.year }}, {{ cookiecutter.owner_name }}'
author = '{{ cookiecutter.owner_name }}'

# The full version, including alpha/beta/rc tags
release = '0.1.0'


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = conf['extensions']

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

# A boolean that decides whether module names are prepended to all object names
# (for object types where a “module” of some kind is defined),
# e.g. for py:function directives. Default is True.
add_module_names = conf['add_module_names']

# -- Auto-doc Options --------------------------------------------------------
autodoc_mock_imports = conf['ext']['autodoc']['autodoc_mock_imports']

# -- FreeCAD Custom Property Table Options -----------------------------------
remove_app_property_prefix_from_type = conf['ext'][
    'freecad_custom_property_table']['remove_app_property_prefix_from_type']

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = conf['html_theme']

html_logo = './_static/ose-sticker-logo.svg'

html_css_files = conf['html_css_files']

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

extlinks = conf['ext']['extlinks']['extlinks']



import os
from pathlib import Path


def handle_env_command(env_subcommand: str) -> None:
    if env_subcommand == 'bootstrap':
        conda_prefix = os.environ.get('CONDA_PREFIX', None)
        if not conda_prefix:
            print('Environment variable "CONDA_PREFIX" not set.')
            print('Is your environment activated?')
            return None
        conda_env_name = os.path.basename(conda_prefix)
        conda_lib = os.path.join(conda_prefix, 'lib')
        conda_etc_base_path = os.path.join(conda_prefix, 'etc', 'conda')

        activate_instructions = '#!/bin/sh\n' + \
            'export INITIAL_PYTHONPATH=${PYTHONPATH}\n' + \
            'export PYTHONPATH={}:${{PYTHONPATH}}\n'.format(conda_lib)
        setup_conda_environment_hook_directory(
            conda_etc_base_path, 'activate.d', activate_instructions)

        deactivate_instructions = '#!/bin/sh\n' + \
            'export PYTHONPATH=${INITIAL_PYTHONPATH}\n' + \
            'unset INITIAL_PYTHONPATH\n'
        setup_conda_environment_hook_directory(
            conda_etc_base_path, 'deactivate.d', deactivate_instructions)

        print('Environment bootstrapped.')
        print('To reactivate your environment, run:\n')
        print('    conda activate {}\n'.format(conda_env_name))


def setup_conda_environment_hook_directory(conda_etc_base_path,
                                           directory,
                                           text):
    directory_path = os.path.join(conda_etc_base_path, directory)
    Path(directory_path).mkdir(parents=True, exist_ok=True)
    env_vars_path = os.path.join(directory_path, 'env_vars.sh')
    Path(env_vars_path).touch(exist_ok=True)
    Path(env_vars_path).write_text(text)



import os
from subprocess import PIPE, Popen


def execute_command(command: str,
                    env: dict = {},
                    exit_on_non_zero_code: bool = True) -> int:
    print('Executing the following command:\n')
    print('    {}\n'.format(command))
    environ = os.environ.copy()
    environ.update(env)
    with Popen(command, stdout=PIPE, bufsize=1, universal_newlines=True, shell=True, env=environ) as process:
        for line in process.stdout:
            print(line, end='')
    if process.returncode != 0 and exit_on_non_zero_code:
        exit(1)
    return process.returncode



import shutil


def check_for_executable_in_path(executable_name) -> None:
    if shutil.which(executable_name) is None:
        print('{} must be installed, and available in your PATH.'.format(
            executable_name))
        exit(0)



import FreeCADGui as Gui

from .command.add_box import AddBoxCommand

#: Command Namespace
command_namespace = '{{ cookiecutter.command_namespace }}'


def register_commands():
    """
    Register all workbench commands,
    and associate them to toolbars, menus, sub-menus, and context menu.
    """
    add_box_key = _register(AddBoxCommand.NAME, AddBoxCommand())

    #: Main Toolbar Commands
    main_toolbar_commands = [
        add_box_key
    ]

    #: Main Menu Commands
    main_menu_commands = [
        add_box_key
    ]

    return main_toolbar_commands, main_menu_commands


def _register(name, command):
    key = '{}_{}'.format(command_namespace, name)
    Gui.addCommand(key, command)
    return key



import os

from .execute_command import execute_command


def handle_docs_command(base_package, root_of_git_repository) -> None:
    path_to_docs_dir = os.path.join(
        root_of_git_repository, 'docs')
    path_to_build_dir = os.path.join(
        path_to_docs_dir, '_build')
    path_to_sphinx_source_dir = os.path.join(
        path_to_docs_dir, base_package)
    execute_command('rm -rf {}'.format(path_to_build_dir))
    execute_command('rm -rf {}'.format(path_to_sphinx_source_dir))
    execute_command(
        'cd {} && sphinx-build -W --keep-going --color . {}'.format(path_to_docs_dir, path_to_build_dir))
    print('To view, open docs/_build/index.html in a web browser, or run:\n')
    print('    osewb browse docs\n')



from setuptools import setup

setup(
    name='{{ cookiecutter.repo_name }}',
    version='0.1.0',
    packages=[
        'freecad',
        'freecad.{{ cookiecutter.base_package }}',
        '{{ cookiecutter.base_package }}'
    ],
    author='{{ cookiecutter.owner_name }}',
    description='A FreeCAD workbench for designing {{ cookiecutter.machine_display_name }} machines by Open Source Ecology (OSE).',
    install_requires=[],
    include_package_data=True,
    classifiers=[
        # Full List: https://pypi.org/pypi?%3Aaction=list_classifiers
        'License :: OSI Approved :: GNU Lesser General Public License v2 or later (LGPLv2+)',
        'Programming Language :: Python :: 3 :: Only'
    ]
)



import unittest

# Need to import FreeCAD
import FreeCAD as App
from {{ cookiecutter.base_package }}.part import Box


class BoxTest(unittest.TestCase):
    """
    Test for Box Part class.
    """

    def test_make(self):
        # Make a 10 x 10 x 10 box
        box = Box.make()
        self.assertEqual(len(box.Faces), 6)
        self.assertAlmostEqual(box.Volume, 1000)
        self.assertAlmostEqual(box.Area, 600)
        self.assertEqual(box.TypeId, 'Part::TopoShape')


if __name__ == '__main__':
    unittest.main()



"""
The ``{{ cookiecutter.base_package }}`` package is:

* Independent of the ``freecad.{{ cookiecutter.base_package }}`` package
* Must not know about the FreeCAD GUI (i.e. ``FreeCADGui`` or ``Gui``)
* Executable from a command-line context
"""



import os
from typing import Union

from .check_for_executable_in_path import check_for_executable_in_path


def find_base_package() -> str:
    repo_root = find_root_of_git_repository()
    if repo_root is None:
        return None
    contents = os.listdir(repo_root)
    directories = [c for c in contents if os.path.isdir(
        os.path.join(repo_root, c)) and c.startswith('ose') and not c.endswith('egg-info')]
    if len(directories) == 0:
        print('No base package starting with "ose" found in repository.')
        return None
    elif len(directories) > 1:
        print('Multiple potential base packages starting with "ose" found:\n')
        print('    {}\n'.format(', '.join(directories)))
        print('Choosing first "{}" as base package.\n'.format(directories[0]))
    return directories[0]


def find_root_of_git_repository() -> Union[str, None]:
    """Find the root of the current git repository.
    Returns None if there's an error, or not in a git repository.

    :return: path to root of git repository
    :rtype: str
    """
    return exec_git_command('git rev-parse --show-toplevel')


def find_git_user_name() -> Union[str, None]:
    """Find the user name defined by git config.

    :return: Git user name
    :rtype: str
    """
    return exec_git_command('git config user.name')


def exec_git_command(git_command) -> Union[str, None]:
    """Find the root of the current git repository.
    Returns None if there's an error, or not in a git repository.

    :param git_command: git command string
    :return: path to root of git repository
    :rtype: str
    """
    check_for_executable_in_path('git')
    pipe = os.popen(git_command)
    output = pipe.read().strip()
    if pipe.close() is not None:
        return None
    return output



"""Shared base configuration for OSE workbench documentation.
"""
import os
import sys

# Add lib folder of conda env to sys.path for building docs on Read the Docs
# and importing FreeCAD
on_read_the_dcs = os.environ.get('READTHEDOCS') == 'True'
if on_read_the_dcs:
    conda_lib_path = os.path.join(
        os.environ['CONDA_ENVS_PATH'], os.environ['CONDA_DEFAULT_ENV'], 'lib')
    sys.path.append(conda_lib_path)

# Configuration for Sphinx Python Documentation Generator
# https://www.sphinx-doc.org/en/master/usage/configuration.html
conf = {
    # Read the Docs Sphinx Theme
    # Designed for great reader experience on both desktop and mobile devices.
    # https://github.com/readthedocs/sphinx_rtd_theme
    'html_theme': 'sphinx_rtd_theme',
    'html_css_files': [
        'https://ose-workbench-platform.readthedocs.io/en/latest/_static/theme_overrides.css'
    ],

    # Whether module names are prepended to all object names
    # (for object types where a “module” of some kind is defined),
    # e.g. for py:function directives. Default is True.
    # https://www.sphinx-doc.org/en/master/usage/configuration.html#confval-add_module_names
    'add_module_names': False,

    # List of strings that are module names of extensions.
    # https://www.sphinx-doc.org/en/master/usage/configuration.html#confval-extensions
    'extensions': [

        # Include documentation from docstrings
        # https://www.sphinx-doc.org/en/master/usage/extensions/autodoc.html
        'sphinx.ext.autodoc',

        # Helps with having many external links that point to the OSE Wiki.
        # https://www.sphinx-doc.org/en/master/usage/extensions/extlinks.html
        'sphinx.ext.extlinks',

        # Enables localization of theme strings in translated output
        'sphinx_rtd_theme',

        # Custom OSE Workbench Sphinx Extensions
        # See respective docstring
        'osewb.docs.ext.freecad_custom_property_table',
        'osewb.docs.ext.freecad_icon'
    ],

    # Base configuration for the above Sphinx extensions
    'ext': {
        'autodoc': {
            # List of modules to be mocked up.
            # Useful when some external dependencies are not met at build time,
            # and break the build process.
            # https://www.sphinx-doc.org/en/master/usage/extensions/autodoc.html#confval-autodoc_mock_imports
            'autodoc_mock_imports': [
                'FreeCADGui'
            ]
        },
        'extlinks': {
            # Dictionary of external sites,
            # mapping unique short alias names to a base URL and a prefix.
            # https://www.sphinx-doc.org/en/master/usage/extensions/extlinks.html#confval-extlinks
            'extlinks': {
                'osewikipage': (
                    'https://wiki.opensourceecology.org/wiki/%s', ''
                ),
                'freecadwikipage': (
                    'https://wiki.freecadweb.org/%s', ''
                )
            }
        },
        'freecad_custom_property_table': {
            'remove_app_property_prefix_from_type': True
        }
    }
}



import os
import re

from docutils import nodes


def fcicon_role(name, rawtext, text, lineno, inliner, options={}, content=[]):
    """FreeCAD Icon.

    Returns 2 part tuple containing list of nodes to insert into the
    document and a list of system messages.  Both are allowed to be
    empty.

    :param name: The role name used in the document.
    :param rawtext: The entire markup snippet, with role.
    :param text: The text marked with the role.
    :param lineno: The line number where rawtext appears in the input.
    :param inliner: The inliner instance that called us.
    :param options: Directive options for customization.
    :param content: The directive content for customization.
    """
    try:
        pattern = re.compile('([\w\s]+) \((sm|md|lg)\) \<(.*\.\w+)\>')
        result = pattern.search(text)
        if not result or len(result.groups()) != 3:
            raise ValueError
        alt, size, filename = result.groups()
    except ValueError:
        msg = inliner.reporter.error(
            'FreeCAD Icon must include alt, size (sm, md, or lg), and filename (e.g. :fcicon:`My Icon Alt (md) <MyIcon.svg>`); '
            '"%s" is invalid.' % text, line=lineno)
        prb = inliner.problematic(rawtext, rawtext, msg)
        return [prb], [msg]
    app = inliner.document.settings.env.app
    try:
        freecad_icon_directory = app.config.freecad_icon_directory
        if not freecad_icon_directory:
            raise AttributeError
    except AttributeError:
        raise ValueError(
            'freecad_icon_directory configuration value is not set')
    image = make_image_node(freecad_icon_directory, alt, size, filename)
    return [image], []


def make_image_node(freecad_icon_directory, alt, size, filename):
    """Create a link to a BitBucket resource.

    :param freecad_icon_directory: Directory to FreeCAD Icons.
    :param alt: Alt text of icon.
    :param size: Must be one of "sm" (small), "md", (medium), or "lg" (large).
    :param filename: Filename of icon.
    """
    dim = {
        'sm': '16px',  # small size, as it appears in the tree view.
        'md': '32px',  # medium size, regular buttons such as toolbars.
        'lg': '64'  # original size, large buttons.
    }[size]
    uri = os.path.join(freecad_icon_directory, filename)
    # Preface uri with forward slash to make path relative to root of docs.
    # TODO: Should '/' be os.path.sep for Windows?
    return nodes.image(uri='/' + uri, alt=alt, width=dim, height=dim)


def setup(app):
    """Install the plugin.

    :param app: Sphinx application context.
    """
    app.add_role('fcicon', fcicon_role)
    app.add_config_value('freecad_icon_directory', None, 'env')



'''
Adds a ``.. fc-custom-property-table::`` directive to create a table documenting the properties of custom FreeCAD objects.

Must add the ``.. fc-custom-property-table::`` directive to the docstring of a scripted object class:

.. code-block:: python

   class BoxModel:
       """
       .. fc-custom-property-table::
       """

Or pass the path to the class as an **optional** argument:

.. code-block:: bash

   .. fc-custom-property-table:: examples.box_model.BoxModel

Supports a ``remove_app_property_prefix_from_type`` configuration value to remove the ``App::Property`` prefix from the **Type**. Defaults to ``False``.

These objects are also known as "FeaturePython Objects" or "Scripted Objects" in the FreeCAD community.

See `BoxModel for an example <examples/examples.html#module-examples.box_model>`_.
'''
import importlib
import re

from docutils import nodes
from sphinx.util import logging
from sphinx.util.docutils import SphinxDirective

import FreeCAD as App

logger = logging.getLogger(__name__)


class FreeCADCustomPropertyTable(SphinxDirective):

    has_content = False
    optional_arguments = 1

    def run(self):
        source_info = self.get_source_info()
        class_path = None
        if len(self.arguments) > 0:
            class_path = self.arguments[0]
        else:
            source = source_info[0]
            pattern = re.compile('.* of (.*)')
            # (e.g. osewb.docs.ext.box_model.BoxModel)
            class_path = pattern.match(source).group(1)
        class_path_parts = class_path.split('.')
        # (e.g. BoxModel)
        class_name = class_path_parts[-1]
        # (e.g. osewb.docs.ext.box_model)
        module_path = '.'.join(class_path_parts[:-1])
        model_module = importlib.import_module(module_path)
        class_ = getattr(model_module, class_name)
        document = App.newDocument()
        obj = document.addObject('Part::FeaturePython', 'FeaturePython')
        feature_python = document.addObject(
            'Part::FeaturePython', 'FeaturePython')
        feature_python_attrs = [x for x in dir(feature_python)]
        remove_app_property_prefix_from_type = self.config.remove_app_property_prefix_from_type
        try:
            class_(obj)
            custom_obj_attrs = [x for x in dir(
                obj) if x not in feature_python_attrs]
            data = _build_rows(custom_obj_attrs, obj, remove_app_property_prefix_from_type)
        except TypeError:
            logger.warning(
                'Unable to instantiate {}.\n'.format(class_name) +
                'Expecting constructor signature:\n' +
                "    obj = document.addObject('Part::FeaturePython', 'FeaturePython')\n" +
                '    {}(obj)\n'.format(class_name) +
                'Defaulting to property table with no rows.', location=source_info)
            data = []
        return _build_property_table(data)


def _pascal_case_to_human_readable(str):
    words = [[str[0]]]

    for c in str[1:]:
        if words[-1][-1].islower() and c.isupper():
            words.append(list(c))
        else:
            words[-1].append(c)

    return ' '.join([''.join(word) for word in words])


def _build_property_table(data):
    header = (
        nodes.paragraph(text='Name'),
        nodes.paragraph(text='Type'),
        nodes.paragraph(text='Default Value'),
        nodes.paragraph(text='Description')
    )
    colwidths = (1, 1, 1, 1)
    table = nodes.table()
    tgroup = nodes.tgroup(cols=len(header))
    table += tgroup
    for colwidth in colwidths:
        tgroup += nodes.colspec(colwidth=colwidth)
    thead = nodes.thead()
    tgroup += thead
    thead += create_table_row(header)
    tbody = nodes.tbody()
    tgroup += tbody
    for data_row in data:
        tbody += create_table_row(data_row)
    return [table]


def _build_rows(custom_obj_attrs, obj, remove_app_property_prefix_from_type):
    rows = []
    for attr in custom_obj_attrs:
        description = obj.getDocumentationOfProperty(attr)
        property_type = obj.getTypeIdOfProperty(attr)
        if remove_app_property_prefix_from_type:
            property_type = property_type.replace('App::Property', '')
        default_value = getattr(obj, attr)
        rows.append((
            nodes.strong(text=_pascal_case_to_human_readable(attr)),
            nodes.literal(text=property_type),
            nodes.paragraph(text=default_value),
            nodes.paragraph(text=description)
        ))
    return rows


def create_table_row(row_cells):
    row = nodes.row()
    for cell in row_cells:
        entry = nodes.entry()
        row += entry
        entry += cell
    return row


def setup(app):
    app.add_directive('fc-custom-property-table', FreeCADCustomPropertyTable)
    app.add_config_value('remove_app_property_prefix_from_type', False, 'env')

    return {
        'version': '0.1.0',
        'parallel_read_safe': True,
        'parallel_write_safe': True,
    }



import os
from pathlib import Path

from isort import SortImports
from jinja2 import Environment, PackageLoader


def handle_make_command(base_package: str,
                        root_of_git_repository: str,
                        make_subcommand: str,
                        name: str) -> None:
    if make_subcommand == 'part':
        env = Environment(
            loader=PackageLoader('osewb', 'templates'))
        template = env.get_template('part.py')
        part_package_path = Path(os.path.join(
            root_of_git_repository,
            base_package,
            'part'
        ))
        part_package_path.mkdir(exist_ok=True)
        part_package_init_module_path = part_package_path.joinpath(
            '__init__.py')
        part_lower = name.lower()
        import_statement = 'from .{} import {}\n'.format(part_lower, name)
        with part_package_init_module_path.open('a') as f:
            f.write(import_statement)
        SortImports(part_package_init_module_path.resolve())
        new_part_package_path = Path(
            part_package_path).joinpath(part_lower)
        new_part_package_path.mkdir(exist_ok=True)
        init_module_path = new_part_package_path.joinpath('__init__.py')
        init_module_path.touch(exist_ok=True)
        with init_module_path.open('a') as f:
            f.write(import_statement)
        SortImports(init_module_path.resolve())
        module_name = '{}.py'.format(part_lower)
        part_module_path = new_part_package_path.joinpath(module_name)
        part_module_existed_before = part_module_path.exists()
        part_module_path.touch(exist_ok=True)
        if part_module_existed_before:
            print('{} already exists. Skipping part class creation.'.format(
                module_name))
            return None
        part_module_path.write_text(template.render(name=name) + '\n')



import io
from os import path

from setuptools import setup

version = {}
with open('osewb/_version.py') as fp:
    exec(fp.read(), version)

current_dir = path.abspath(path.dirname(__file__))
with io.open(path.join(current_dir, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='ose-workbench-platform',
    description='Common platform for developing Open Source Ecology (OSE) workbenches.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/gbroques/ose-workbench-platform',
    author='G Roques',
    version=version['__version__'],
    packages=['osewb', 'osewb.docs', 'osewb.docs.ext'],
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'osewb = osewb.osewb:main'
        ]
    },
    install_requires=[
        # -----------------------
        # | Source Requirements |
        # -----------------------
        'cookiecutter==1.7.2',
        'Jinja2<3.0.0',  # Also a requirement of cookiecutter
        'jinja2-time>=0.2.0',  # Also a requirement of cookiecutter

        # ---------------------------
        # | Docs Requirements        |
        # | Also in environment.yaml |
        # ----------------------------
        'sphinx==3.1.1',
        'sphinx-rtd-theme==0.5.0',

        # ---------------------
        # | Test Requirements |
        # ---------------------
        'coverage==5.1',
        'coveralls==2.0.0',
        'pytest==5.4.3',
        'pytest-cov==2.9.0',

        # ---------------------
        # | Lint Requirements |
        # ---------------------
        'flake8==3.8.3',
        'flake8-colors==0.1.6',
        'flake8-isort==3.0.0',
        'isort==4.3.21',
        'autopep8==1.5.3',
        'mypy==0.781'
    ],
    classifiers=[
        # Full List: https://pypi.org/pypi?%3Aaction=list_classifiers
        'License :: OSI Approved :: GNU Lesser General Public License v2 or later (LGPLv2+)',
        'Programming Language :: Python :: 3 :: Only'
    ]
)



import argparse
from typing import Tuple

from ._version import __version__
from .find_base_package import find_base_package, find_root_of_git_repository
from .handle_browse_command import handle_browse_command
from .handle_docs_command import handle_docs_command
from .handle_env_command import handle_env_command
from .handle_init_command import handle_init_command
from .handle_lint_command import handle_lint_command
from .handle_make_command import handle_make_command
from .handle_test_command import handle_test_command


def main() -> None:
    command, args = _parse_command()
    if command == 'init':
        machine_display_name = args['machine_display_name']
        handle_init_command(machine_display_name)
    elif command == 'test' or command == 'docs' or command == 'make':
        root_of_git_repository = find_root_of_git_repository()
        if root_of_git_repository is None:
            return None
        base_package = find_base_package()
        if base_package is None:
            return None
        if command == 'test':
            handle_test_command(base_package,
                                root_of_git_repository,
                                with_coverage=args['coverage'])
        elif command == 'docs':
            handle_docs_command(base_package, root_of_git_repository)
        elif command == 'make':
            make_subcommand = args['make_command']
            name = args['name']
            handle_make_command(base_package,
                                root_of_git_repository,
                                make_subcommand,
                                name)
    elif command == 'env':
        env_subcommand = args['env_command']
        handle_env_command(env_subcommand)
    elif command == 'browse' or command == 'lint':
        root_of_git_repository = find_root_of_git_repository()
        if root_of_git_repository is None:
            return None
        if command == 'browse':
            browse_subcommand = args['browse_command']
            handle_browse_command(root_of_git_repository, browse_subcommand)
        elif command == 'lint':
            should_fix = args['fix']
            handle_lint_command(root_of_git_repository, should_fix)


def _parse_command() -> Tuple[str, dict]:
    parser = argparse.ArgumentParser(
        description='A collection commands for OSE workbench development.',
        usage='osewb <command> [<args>]\n')
    parser.add_argument('--version', action='version', version=__version__)
    subparsers = parser.add_subparsers(title='Commands',
                                       dest='command',
                                       required=True)
    make_parser = subparsers.add_parser('make',
                                        help='Commands for making new code',
                                        usage='osewb make <command>')
    make_subparser = make_parser.add_subparsers(title='Commands',
                                                dest='make_command',
                                                required=True)
    part_subparser = make_subparser.add_parser('part',
                                               help='Make Part class',
                                               usage='osewb make part <name>')
    part_subparser.add_argument('name', help='Name for the part class')
    env_parser = subparsers.add_parser('env',
                                       help='Commands for interacting with environments',
                                       usage='osewb env <command>')
    env_subparser = env_parser.add_subparsers(title='Commands',
                                              dest='env_command',
                                              required=True)
    env_subparser.add_parser('bootstrap',
                             help='Bootstrap environment',
                             usage='osewb env bootstrap')
    test_parser = subparsers.add_parser('test',
                                        help='Run tests in workbench',
                                        usage='osewb test')
    test_parser.add_argument('-c', '--coverage',
                             action='store_true',
                             help='Run tests with coverage, and generate report')
    subparsers.add_parser('docs',
                          help='Make documentation',
                          usage='osewb docs')
    lint_parser = subparsers.add_parser('lint',
                                        help='Lint code',
                                        usage='osewb lint')
    lint_parser.add_argument('-f', '--fix',
                             action='store_true',
                             help='Attempt to automatically fix linter issues')
    init_parser = subparsers.add_parser('init',
                                        help='Initialize new workbench',
                                        usage='osewb init <machine_display_name>')
    init_parser.add_argument('machine_display_name',
                             type=str,
                             help='Name of machine in title-case. Surround in double-quotes if name contains spaces (e.g. "CEB Brick Press")')
    browse_parser = subparsers.add_parser('browse',
                                          help='Commands for opening documents in a web browser',
                                          usage='osewb browse <command>')
    browse_subparser = browse_parser.add_subparsers(title='Commands',
                                                    dest='browse_command',
                                                    required=True)
    browse_subparser.add_parser('docs',
                                help='Opens docs in web browser',
                                usage='osewb browse docs')
    browse_subparser.add_parser('coverage',
                                help='Opens coverage report in web browser',
                                usage='osewb browse coverage')
    args = vars(parser.parse_args())
    command = args.pop('command')
    return command, args


if __name__ == '__main__':
    main()



import os

from cookiecutter.main import cookiecutter

from ._version import __version__
from .find_base_package import find_git_user_name


def handle_init_command(machine_display_name: str) -> None:
    dir_path = os.path.dirname(os.path.realpath(__file__))
    git_user_name = find_git_user_name()
    if not git_user_name:
        print('Must configure your git user name:\n')
        print('    git config --global user.name "FIRST_NAME LAST_NAME"\n')
        print('This is used for the owner name throughout the workbench.')
        return
    cookiecutter(os.path.join(dir_path, 'cookiecutter_ose_workbench'),
                 no_input=True,
                 extra_context={
        'machine_display_name': machine_display_name,
        'owner_name': git_user_name,
        'ose_workbench_platform_version': __version__})
    slugified_machine_name = machine_display_name.lower().replace(' ', '-')
    repo_name = 'ose-{}-workbench'.format(slugified_machine_name)
    print('Workbench initialized in "{}" directory.\n'.format(repo_name))
    print('Next, change directories and initialize the git repository:\n'.format(
        repo_name))
    print('    cd {} && git init\n'.format(repo_name))
